@@ -1,5 +1,5 @@
 /* -*- coding: utf-8 -*- */
-/* Copyright (C) 2011-2020  José Luis García Pallero, <jgpallero@gmail.com>
+/* Copyright (C) 2011-2022  José Luis García Pallero, <jgpallero@gmail.com>
  *
  * This file is part of OctCLIP.
  *
@@ -21,7 +21,7 @@
 /******************************************************************************/
 #define HELPTEXT "\
 -*- texinfo -*-\n\
-@deftypefn {}{[@var{p},@var{pp},@var{ni},@var{np}] =}_oc_polybool(@var{sub},@var{clip},@var{op},@var{hor})\n\
+@deftypefn {Function file}{[@var{p},@var{pp},@var{ni},@var{np}] =}_oc_polybool(@var{sub},@var{clip},@var{op},@var{hor})\n\
 \n\
 @cindex Performs boolean operations between two polygons.\n\
 \n\
@@ -72,7 +72,7 @@ polygons they are separated by rows os (NaN,NaN) values.\n\n\
 @var{pp} is a three-column matrix with a number of rows equal to the number\n\
 of polygons stored in the matrix @var{p}. The first column stores the row of\n\
 @var{p} where the corresponding polygon starts, the second column the row of\n\
-@var{p} where the polygon end, and the third colum is a mark identifying if\n\
+@var{p} where the polygon end, and the third column is a mark identifying if\n\
 the polygon is a hole (value 0) or not (value 1). The values of the third\n\
 column are relevant only in the case of the OR operation\n\n\
 @var{ni} is the number of intersections between @var{sub} and @var{clip}.\n\n\
@@ -113,7 +113,7 @@ DEFUN_DLD(_oc_polybool,args,,HELPTEXT)
                 "Incorrect number of input arguments\n\t"
                 "See help _oc_polybool");
         //error message
-        error(errorText);
+        error("%s",errorText);
     }
     else
     {
@@ -161,7 +161,7 @@ DEFUN_DLD(_oc_polybool,args,,HELPTEXT)
             //error text
             sprintf(&errorText[strlen(errorText)],"Error in memory allocation");
             //error message
-            error(errorText);
+            error("%s",errorText);
             //exit
             return outputList;
         }
@@ -199,7 +199,7 @@ DEFUN_DLD(_oc_polybool,args,,HELPTEXT)
                     "The third input argument (op=%s) is not correct",
                     opchar.c_str());
             //error message
-            error(errorText);
+            error("%s",errorText);
             //exit
             return outputList;
         }
@@ -217,7 +217,7 @@ DEFUN_DLD(_oc_polybool,args,,HELPTEXT)
             //error text
             sprintf(&errorText[strlen(errorText)],"Error in memory allocation");
             //error message
-            error(errorText);
+            error("%s",errorText);
             //exit
             return outputList;
         }
@@ -260,12 +260,15 @@ DEFUN_DLD(_oc_polybool,args,,HELPTEXT)
         //output matrix
         Matrix p(nElem,2);
         //copy output data
-        pos = 0;
-        for(i=posStart;i<=posEnd;i++)
+        if(nElem!=0)
         {
-            p(pos,0) = result->x[i];
-            p(pos,1) = result->y[i];
-            pos++;
+            pos = 0;
+            for(i=posStart;i<=posEnd;i++)
+            {
+                p(pos,0) = result->x[i];
+                p(pos,1) = result->y[i];
+                pos++;
+            }
         }
         ////////////////////////////////////////////////////////////////////////
         ////////////////////////////////////////////////////////////////////////
