@@ -8,12 +8,12 @@
 \note Este fichero contiene funciones paralelizadas con OpenMP.
 \date 10 de octubre de 2009
 \version 1.0
-\section Licencia Licencia
-Copyright (c) 2009-2011, José Luis García Pallero. All rights reserved.
-
+\copyright
+Copyright (c) 2009-2020, José Luis García Pallero. All rights reserved.
+\par
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
-
+\par
 - Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this
@@ -22,7 +22,7 @@ are permitted provided that the following conditions are met:
 - Neither the name of the copyright holders nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.
-
+\par
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -267,7 +267,7 @@ double Minimo(const double* lista,
     //en versiones anteriores no existe la posibilidad de usar reduction(min:)
 #if defined(_OPENMP)&&(_OPENMP>=GEOC_OMP_F_3_1)
 #pragma omp parallel for default(none) \
- shared(lista) \
+ shared(nDatos,incDatos,lista) \
  private(i,pos) \
  reduction(min:salida)
 #endif
@@ -306,7 +306,7 @@ double Maximo(const double* lista,
     //en versiones anteriores no existe la posibilidad de usar reduction(max:)
 #if defined(_OPENMP)&&(_OPENMP>=GEOC_OMP_F_3_1)
 #pragma omp parallel for default(none) \
- shared(lista) \
+ shared(nDatos,incDatos,lista) \
  private(i,pos) \
  reduction(max:salida)
 #endif
@@ -345,7 +345,7 @@ double MinimoAbs(const double* lista,
     //en versiones anteriores no existe la posibilidad de usar reduction(min:)
 #if defined(_OPENMP)&&(_OPENMP>=GEOC_OMP_F_3_1)
 #pragma omp parallel for default(none) \
- shared(lista) \
+ shared(nDatos,incDatos,lista) \
  private(i,pos) \
  reduction(min:salida)
 #endif
@@ -384,7 +384,7 @@ double MaximoAbs(const double* lista,
     //en versiones anteriores no existe la posibilidad de usar reduction(max:)
 #if defined(_OPENMP)&&(_OPENMP>=GEOC_OMP_F_3_1)
 #pragma omp parallel for default(none) \
- shared(lista) \
+ shared(nDatos,incDatos,lista) \
  private(i,pos) \
  reduction(max:salida)
 #endif
@@ -416,14 +416,18 @@ size_t MinimoSizeT(const size_t* lista,
     //variable de posición
     size_t pos=0;
     //variable de salida, inicializada como el máximo valor para un size_t
+#if defined(SIZE_MAX)
     size_t salida=SIZE_MAX;
+#else
+    size_t salida=(size_t)ULONG_MAX;
+#endif
     ////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////
     //paralelización con OpenMP, sólo si la versión es superior a la 3.0
     //en versiones anteriores no existe la posibilidad de usar reduction(min:)
 #if defined(_OPENMP)&&(_OPENMP>=GEOC_OMP_F_3_1)
 #pragma omp parallel for default(none) \
- shared(lista) \
+ shared(nDatos,incDatos,lista) \
  private(i,pos) \
  reduction(min:salida)
 #endif
@@ -462,7 +466,7 @@ size_t MaximoSizeT(const size_t* lista,
     //en versiones anteriores no existe la posibilidad de usar reduction(max:)
 #if defined(_OPENMP)&&(_OPENMP>=GEOC_OMP_F_3_1)
 #pragma omp parallel for default(none) \
- shared(lista) \
+ shared(nDatos,incDatos,lista) \
  private(i,pos) \
  reduction(max:salida)
 #endif
