@@ -9,7 +9,7 @@
 \author José Luis García Pallero, jgpallero@gmail.com
 \date 14 de mayo de 2011
 \copyright
-Copyright (c) 2011-2020, José Luis García Pallero. All rights reserved.
+Copyright (c) 2011-2022, José Luis García Pallero. All rights reserved.
 \par
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
@@ -1799,12 +1799,13 @@ polig* CreaPoligPoliClip(vertPoliClip* poli,
     //asignamos memoria para los vectores de coordenadas de la estructura
     result->x = (double*)malloc(nElem*sizeof(double));
     result->y = (double*)malloc(nElem*sizeof(double));
-    //asignamos memoria para los vectores de posición
+    //asignamos memoria para los vectores de posición y de atributos
     result->posIni = (size_t*)malloc(sizeof(size_t));
     result->nVert = (size_t*)malloc(sizeof(size_t));
+    result->atr = (int*)malloc(sizeof(size_t));
     //comprobamos los posibles errores de asignación de memoria
     if((result->x==NULL)||(result->y==NULL)||(result->posIni==NULL)||
-       (result->nVert==NULL))
+       (result->nVert==NULL)||(result->atr==NULL))
     {
         //liberamos la posible memoria asignada
         LibMemPolig(result);
@@ -1822,6 +1823,8 @@ polig* CreaPoligPoliClip(vertPoliClip* poli,
     //asignamos la posición de inicio y el número de vértices
     result->posIni[0] = 1;
     result->nVert[0] = nVert;
+    //inicializamos el atributo a cero
+    result->atr[0] = 0;
     //asignamos los separadores de polígono al principio y al final
     result->x[0] = GeocNan();
     result->y[0] = GeocNan();
