@@ -8,12 +8,12 @@
 \author José Luis García Pallero, jgpallero@gmail.com
 \note Este fichero contiene funciones paralelizadas con OpenMP.
 \date 20 de abril de 2011
-\section Licencia Licencia
-Copyright (c) 2011, José Luis García Pallero. All rights reserved.
-
+\copyright
+Copyright (c) 2011-2020, José Luis García Pallero. All rights reserved.
+\par
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
-
+\par
 - Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this
@@ -22,7 +22,7 @@ are permitted provided that the following conditions are met:
 - Neither the name of the copyright holders nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.
-
+\par
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -43,11 +43,15 @@ ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include<stdio.h>
 #include<stdlib.h>
 #include<math.h>
+#include"libgeoc/arco.h"
+#include"libgeoc/constantes.h"
 #include"libgeoc/dpeucker.h"
+#include"libgeoc/dpeuckera.h"
 #include"libgeoc/errores.h"
 #include"libgeoc/fgeneral.h"
 #include"libgeoc/geocnan.h"
 #include"libgeoc/geocomp.h"
+#include"libgeoc/proyecaux.h"
 /******************************************************************************/
 /******************************************************************************/
 #ifdef __cplusplus
@@ -62,6 +66,7 @@ extern "C" {
 \date 26 de mayo de 2011: Reorganización total de la estructura.
 \date 28 de mayo de 2011: Adición de los campos polig::hayArea, polig::area,
       polig::hayLim, polig::xMin, polig::xMax, polig::yMin e polig::yMax.
+\date 10 de mayo de 2020: Adición del campo polig::atr.
 */
 typedef struct
 {
@@ -156,6 +161,13 @@ typedef struct
              contrario al de las agujas del reloj.
     */
     double* area;
+    /**
+    \brief Vector de polig::nPolig elementos, que almacena un atributo
+           cualquiera elegido por el usuario referido a cada polígono. Este
+           campo sólo sirve para posibles manejos del usuario en el empleo de
+           la estructura. Los elementos se inicializan con el valor 0.
+    */
+    int* atr;
 }polig;
 /******************************************************************************/
 /******************************************************************************/
@@ -190,8 +202,7 @@ polig* IniciaPoligVacio(void);
 /**
 \brief Función auxiliar para la rutina de creación de una estructura \ref polig
        a partir de dos vectores que contienen las coordenadas de los vértices.
-
-       Esta función calcula el número máximo de elementos que almacenarán los
+\brief Esta función calcula el número máximo de elementos que almacenarán los
        vectores de coordenadas de una estructura \ref polig y el número de
        polígonos almacenados en los vectores de trabajo.
 \param[in] nElem Número de elementos de los vectores de coordenadas originales.
@@ -231,8 +242,7 @@ int AuxCreaPolig1(const size_t nElem,
 /**
 \brief Función auxiliar para la rutina de creación de una estructura \ref polig
        a partir de dos vectores que contienen las coordenadas de los vértices.
-
-       Esta función copia una serie de datos de dos vectores en otros dos.
+\brief Esta función copia una serie de datos de dos vectores en otros dos.
 \param[in] x Vector que contiene las coordenadas X de los vértices a copiar.
 \param[in] y Vector que contiene las coordenadas Y de los vértices a copiar.
 \param[in] nElem Número de elementos de los vectores \em x e \em y.
@@ -269,8 +279,7 @@ void AuxCreaPolig2(const double* x,
 /**
 \brief Función auxiliar para las rutinas de creación de estructuras \ref polig
        a partir de dos vectores que contienen las coordenadas de los vértices.
-
-       Esta función crea los polígonos en el formato de almacenamiento de
+\brief Esta función crea los polígonos en el formato de almacenamiento de
        \ref polig a partir de los vectores de entrada.
 \param[in] x Vector que contiene las coordenadas X de los vértices de trabajo.
 \param[in] y Vector que contiene las coordenadas Y de los vértices de trabajo.
@@ -350,7 +359,8 @@ void AuxCreaPolig3(const double* x,
       \em incY.
 \note Si los vectores \em x e \em y almacenan varios polígonos, éstos se separan
       mediante valores #GEOC_NAN. Poner #GEOC_NAN en la primera posición y/o la
-      última es opcional.
+      última es opcional, pero sea cual sea la elección ha de ser la misma para
+      los dos vectores.
 \note Los posibles valores #GEOC_NAN han de estar en las mismas posiciones en
       \em x e \em y.
 \note Para los polígonos, es opcional repetir las coordenadas del primer vértice
@@ -624,20 +634,49 @@ void LimitesPoligonosPolig(const double* x,
 \param[in,out] poli Estructura \ref polig, que almacena una serie de polígonos.
                Al término de la ejecución de la función, se han añadido las
                superficies de los polígonos almacenados.
+\param[in] facCoor Factor a aplicar a las coordenadas antes del cálculo de las
+           superficies. Si los cálculos se van a hacer sobre el elipsoide, este
+           argumento debe llevar las coordenadas a radianes, si no lo están.
+\param[in] geo Identificador de que el polígono está sobre la superficie del
+           elipsoide. Dos posibilidades:
+           - 0: El polígono está en el plano euclideo.
+           - Distinto de 0: El polígono está sobre la superficie del elipsoide.
+\param[in] a Semieje mayor del elipsoide. Este argumento sólo es tenido en
+           cuenta si se ha indicado trabajo sobre el elipsoide mediante el
+           argumento \em geo.
+\param[in] f Aplanamiento del elipsoide. Este argumento sólo es tenido en
+           cuenta si se ha indicado trabajo sobre el elipsoide mediante el
+           argumento \em geo.
 \return Variable de error. Dos posibilidades:
         - #GEOC_ERR_NO_ERROR: Todo ha ido bien.
         - #GEOC_ERR_ASIG_MEMORIA: Ha ocurrido un error de asignación de memoria.
 \note Esta función asume que la estructura de entrada \ref polig tiene memoria
       asignada.
+\note El área calculada sólo es correcta si el polígono es simple, esto es, si
+      sus lados no se cortan entre ellos mismos (ver la documentación de
+      polig::area).
+\note Se asume que \em facCoor>0.0, condición que no se controla internamente.
 \note En caso de error de asignación de memoria, la memoria de la estructura de
       entrada no se libera.
+\note El cálculo del área sobre el elipsoide se realiza proyectando los puntos
+      mediante una proyección cilíndrica equivalente de Lambert.
 \date 29 de mayo de 2011: Creación de la función.
 \date 01 de agosto de 2011: Corregido error que hacía que se calculasen mal las
       superficies debido a un incorrecto uso del argumento \em restaPosIni de la
       función \ref AreaPoligonosSimplesPolig.
+\date 16 de agosto de 2013: Adición de la capacidad de cálculo de áreas para
+      polígonos sobre la superficie del elipsoide.
+\date 17 de agosto de 2013: Modificación de la función para que el área se
+      calcule siempre, aunque la estructura ya la contenga. Se dota de esta
+      capacidad por si se llama repetidas veces a la función con valores
+      distintos de \em facCoor.
 \note Esta función todavía no está probada.
 */
-int CalcAreaPolig(polig* poli);
+int CalcAreaPolig(polig* poli,
+                  const double facCoor,
+                  const int geo,
+                  const double a,
+                  const double f);
 /******************************************************************************/
 /******************************************************************************/
 /**
@@ -893,61 +932,110 @@ void MuevePolig(polig* poli,
                Al término de la ejecución de la función, almacena el resultado
                de la aplicación a cada polígono del algoritmo de aligerado de
                vértices implementado en la función \ref AligeraPolilinea.
-\param[in] tol Tolerancia para eliminar vértices, en las mismas unidades que las
-           coordenadas de los vértices. Ver la ayuda de la función
-           \ref AligeraPolilinea.
+\param[in] esf Identificador de que el polígono está sobre la superficie de la
+           esfera. Dos posibilidades:
+           - 0: El polígono está en el plano euclideo.
+           - Distinto de 0: El polígono está sobre la esfera.
+\param[in] facCoor Factor de escala a aplicar a las coordenadas del polígono
+           para realizar el aligerado. Si se trabaja sobre la esfera, este
+           argumento ha de llevar las coordenadas de los puntos de trabajo a
+           radianes. Las coordenadas devueltas al término de la ejecución de
+           está función no se verán afectadas por este factor.
+\param[in] tol Tolerancia para eliminar vértices. Si se trabaja sobre la
+           superficie de la esfera, este valor ha de estar en radianes, es
+           decir, será una distancia sobre la superficie de la esfera de radio
+           unidad. Ver la ayuda de la función \ref AligeraPolilinea.
+\param[in] paralelizaTol Identificador para evaluar o no en paralelo si los
+           puntos candidatos están en tolerancia. Dos posibilidades:
+           - 0: Se evalúa en serie (aunque la compilación se haya hecho en
+             paralelo) si los puntos están en tolerancia.
+           - Distinto de 0: Se evalúa en paralelo (sólo si se ha compilado en
+             paralelo) si los puntos están en tolerancia.
 \param[in] robusto Identificador para realizar o no un aligerado robusto. Ha de
            ser un elemento del tipo enumerado #GEOC_DPEUCKER_ROBUSTO. Varias
            posibilidades:
-           - #GeocDPeuckerRobNo: No se aplica el algoritmo robusto.
+           - #GeocDPeuckerOriginal: Utiliza el algoritmo de Douglas-Peucker
+             original, que no es robusto.
+           - #GeocDPeuckerRobNo: Utiliza la variación no recursiva del algoritmo
+             de Douglas-Peucker, que no es robusta.
            - #GeocDPeuckerRobSi: Se aplica el algoritmo robusto completo, que
-             garantiza la no ocurrencia de auto intersecciones en el polígono
+             garantiza la no ocurrencia de auto intersecciones en la polilínea
              resultante. Internamente, primero se aplica el tratamiento robusto
              de la opción #GeocDPeuckerRobOrig y luego el de la opción
              #GeocDPeuckerRobAuto.
            - #GeocDPeuckerRobOrig: Se aplica un algoritmo semi robusto que
-             consiste en garantizar que los segmentos del polígono aligerado que
-             se van creando no intersectarán con ninguno de los segmentos que
-             forman los vértices que quedan por procesar del polígono original.
-             En casos muy especiales, este algoritmo puede seguir dando lugar a
-             auto intersecciones.
+             consiste en garantizar que los segmentos de la polilínea aligerada
+             que se van creando no intersectarán con ninguno de los segmentos
+             que forman los vértices que quedan por procesar de la polilínea
+             original. En casos muy especiales, este algoritmo puede seguir
+             dando lugar a auto intersecciones.
            - #GeocDPeuckerRobAuto: Se aplica un algoritmo semi robusto que
-             consiste en garantizar que los segmentos del polígono aligerado que
-             se van creando no intersectarán con ninguno de los segmentos del
-             polígono aligerado creados con anterioridad. En casos muy
+             consiste en garantizar que los segmentos de la polilínea aligerada
+             que se van creando no intersectarán con ninguno de los segmentos de
+             la polilínea aligerada creados con anterioridad. En casos muy
              especiales, este algoritmo puede seguir dando lugar a auto
              intersecciones.
-\param[in] nPtosRobusto Número de puntos del polígono original a utilizar en el
-           caso de tratamiento robusto con las opciones #GeocDPeuckerRobSi o
-           #GeocDPeuckerRobOrig. Si se pasa el valor 0, se utilizan todos los
-           puntos hasta el final del polígono original.
-\param[in] nSegRobusto Número de segmentos del polígono aligerado a utilizar en
-           el caso de tratamiento robusto con las opciones #GeocDPeuckerRobSi o
-           #GeocDPeuckerRobAuto. Si se pasa el valor 0, se utilizan todos los
-           segmentos hasta el inicio del polígono aligerado.
+\param[in] nSegRobOrig Número de segmentos/arcos de la polilínea original a
+           utilizar en el caso de tratamiento robusto con las opciones
+           #GeocDPeuckerRobSi o #GeocDPeuckerRobOrig. Si se pasa el valor 0, se
+           utilizan todos los segmentos/arcos hasta el final de la polilínea
+           original.
+\param[in] nSegRobAuto Número de segmentos de la polilínea aligerada a utilizar
+           en el caso de tratamiento robusto con las opciones #GeocDPeuckerRobSi
+           o #GeocDPeuckerRobAuto. Si se pasa el valor 0, se utilizan todos los
+           segmentos hasta el inicio de la polilínea aligerada.
+\param[in] a Semieje mayor del elipsoide. Este argumento sólo es tenido en
+           cuenta si se ha indicado trabajo sobre el elipsoide mediante el
+           argumento \em geo.
+\param[in] f Aplanamiento del elipsoide. Este argumento sólo es tenido en
+           cuenta si se ha indicado trabajo sobre el elipsoide mediante el
+           argumento \em geo.
 \return Variable de error. Dos posibilidades:
         - #GEOC_ERR_NO_ERROR: Todo ha ido bien.
         - #GEOC_ERR_ASIG_MEMORIA: Ha ocurrido un error de asignación de memoria.
 \note Esta función asume que \em poli está, como mínimo, inicializado.
 \note Si \em poli tiene límites y/o áreas calculadas en la entrada, también los
       tendrá en la salida.
-\note Un polígono aligerado sólo será válido si después de aplicarle la función
-      \ref AligeraPolilinea mantiene un mínimo de 3 puntos no alineados por lo
-      que, al término de la ejecución de esta función, el resultado puede ser
-      una estructura \ref polig vacía.
+\note Si \em poli está sobre la superficie de la esfera, el campo polig::x
+      almacenará la longitud, mientras que polig::y almacenará la latitud. Del
+      mismo modo, los límites polig::xMin y polig::xMax almacenarán longitudes y
+      polig::yMin y polig::yMax, latitudes.
+\note Se asume que \em facCoor>0.0, condición que no se controla internamente.
+\note Un polígono aligerado sólo será válido si después de aplicarle la
+      función \ref AligeraPolilinea mantiene un mínimo de 3 puntos no alineados,
+      por lo que, al término de la ejecución de esta función, el resultado puede
+      ser una estructura \ref polig vacía.
+\note El argumento \em paralelizaTol \b SÓLO afecta a la paralelización de la
+      comprobación de puntos en tolerancia. Los chequeos de intersección de
+      segmentos/arcos siempre se hacen en paralelo (si el código ha sido
+      compilado al efecto).
 \date 09 de julio de 2011: Creación de la función.
 \date 10 de julio de 2011: Cambio del tipo del argumento \em robusto al tipo
       enumerado #GEOC_DPEUCKER_ROBUSTO.
 \date 31 de julio de 2011: Corregido error con índices a la hora de guardar los
       resultados y comprobación de que los polígonos de salida no estén
       compuestos por tres puntos alineados.
+\date 25 de mayo de 2012: Adición de la posibilidad de usar el algoritmo de
+      Douglas-Peucker original.
+\date 16 de agosto de 2013: Adición de la capacidad de trabajar con polígonos en
+      la superficie de la esfera.
+\date 20 de agosto de 2013: Sustitución de las antiguas variables de entrada
+      \em nPtosRobusto y \em nSegRobusto por \em nSegRobOrig y \em nSegRobAuto.
+\date 23 de agosto de 2013: Adición del argumento de entrada \em paralelizaTol.
+\date 21 de septiembre de 2013: Adición de la capacidad de trabajar sobre la
+      esfera con el algoritmo de Douglas-Peucker original.
 \todo Esta función todavía no está probada.
 */
 int AligeraPolig(polig* poli,
+                 const int esf,
+                 const double facCoor,
                  const double tol,
+                 const int paralelizaTol,
                  const enum GEOC_DPEUCKER_ROBUSTO robusto,
-                 const size_t nPtosRobusto,
-                 const size_t nSegRobusto);
+                 const size_t nSegRobOrig,
+                 const size_t nSegRobAuto,
+                 const double a,
+                 const double f);
 /******************************************************************************/
 /******************************************************************************/
 /**
@@ -960,16 +1048,23 @@ int AligeraPolig(polig* poli,
            del polígono de trabajo. Dos posibles valores:
            - 0: No se imprimen.
            - Distinto de 0: Sí se imprimen.
-\param[in] formCoor Cadena de carácteres indicadora del formato para escribir
+\param[in] formCoor Cadena de caracteres indicadora del formato para escribir
            las coordenadas de los límites. Este argumento sólo se usa
            internamente si se ha indicado la impresión de límites.
 \param[in] impArea Identificador para imprimir o no la superficie del polígono
            de trabajo. Dos posibles valores:
            - 0: No se imprime.
            - Distinto de 0: Sí se imprime.
-\param[in] formArea Cadena de carácteres indicadora del formato para escribir el
+\param[in] formArea Cadena de caracteres indicadora del formato para escribir el
            valor de la superficie. Este argumento sólo se usa internamente si se
            ha indicado la impresión de la superficie del polígono.
+\param[in] impAtr Identificador para imprimir o no el atributo almacenado en
+           polig::atr. Dos posibles valores:
+           - 0: No se imprime.
+           - Distinto de 0: Sí se imprime.
+\param[in] formAtr Cadena de caracteres indicadora del formato para escribir el
+           valor del atributo. Este argumento sólo se usa internamente si se
+           ha indicado la impresión del atributo.
 \param[in] factorX Factor para multiplicar las coordenadas X de los vértices
            antes de imprimirlas.
 \param[in] factorY Factor para multiplicar las coordenadas Y de los vértices
@@ -983,9 +1078,12 @@ int AligeraPolig(polig* poli,
 \param[in] idFich Identificador de fichero abierto para escribir.
 \note Esta función está paralelizada con OpenMP.
 \note La cabecera completa tiene el siguiente formato:
-      <tt>iniCab númVert área xMín xMáx yMín yMáx</tt>.
+      <tt>iniCab númVert área xMín xMáx yMín yMáx atributo</tt>.
 \note Si la estructura no tiene información de límites y/o áreas y se indica que
       se impriman, los valores se calculan internamente.
+\note El área imprimida sólo es correcta si el polígono es simple, esto es, si
+      sus lados no se cortan entre ellos mismos (ver la documentación de
+      polig::area).
 \note Esta función asume que \em poli es una estructura \ref polig correctamente
       almacenada.
 \note Esta función no comprueba si la estructura pasada tiene memoria asignada.
@@ -994,6 +1092,7 @@ int AligeraPolig(polig* poli,
 \note Esta función no comprueba internamente si el identificador pasado
       corresponde a un fichero abierto para escribir.
 \date 18 de junio de 2011: Creación de la función.
+\date 10 de mayo de 2020: Adición de los argumentos \em impAtr y \em formAtr.
 \note Esta función todavía no está probada.
 */
 void ImprimeCabeceraPoligFichero(const polig* poli,
@@ -1003,6 +1102,8 @@ void ImprimeCabeceraPoligFichero(const polig* poli,
                                  const char formCoor[],
                                  const int impArea,
                                  const char formArea[],
+                                 const int impAtr,
+                                 const char formAtr[],
                                  const double factorX,
                                  const double factorY,
                                  const int repitePrimerPunto,
@@ -1029,7 +1130,7 @@ void ImprimeCabeceraPoligFichero(const polig* poli,
            polígonos (\p NaN) detrás del último polígono. Dos posibles valores:
            - 0: No se imprime.
            - Distinto de 0: Sí se imprime.
-\param[in] formCoor Cadena de carácteres indicadora del formato de cada
+\param[in] formCoor Cadena de caracteres indicadora del formato de cada
            coordenada a imprimir.
 \param[in] impCabecera Identificador para imprimir o no una cabecera con
            información general por cada polígono. Dos posibles valores:
@@ -1044,14 +1145,24 @@ void ImprimeCabeceraPoligFichero(const polig* poli,
            de los polígonos de trabajo. Dos posibles valores:
            - 0: No se imprime.
            - Distinto de 0: Sí se imprime.
-\param[in] formArea Cadena de carácteres indicadora del formato para escribir el
+\param[in] formArea Cadena de caracteres indicadora del formato para escribir el
            valor de la superficie. Este argumento sólo se usa internamente si se
            ha indicado la impresión de la superficie de los polígonos.
+\param[in] impAtr Identificador para imprimir o no el atributo almacenado en
+           polig::atr. Dos posibles valores:
+           - 0: No se imprime.
+           - Distinto de 0: Sí se imprime.
+\param[in] formAtr Cadena de caracteres indicadora del formato para escribir el
+           valor del atributo. Este argumento sólo se usa internamente si se
+           ha indicado la impresión del atributo.
 \param[in] idFich Identificador de fichero abierto para escribir.
 \note La cabecera completa tiene el siguiente formato:
-      <tt>iniCab númVert área xMín xMáx yMín yMáx</tt>.
+      <tt>iniCab númVert área xMín xMáx yMín yMáx atributo</tt>.
 \note Si la estructura no tiene información de límites y/o áreas y se indica que
       se impriman, los valores se calculan internamente.
+\note El área imprimida sólo es correcta si el polígono es simple, esto es, si
+      sus lados no se cortan entre ellos mismos (ver la documentación de
+      polig::area).
 \note Esta función asume que \em poli es una estructura \ref polig correctamente
       almacenada.
 \note Esta función no comprueba si la estructura pasada tiene memoria asignada.
@@ -1066,6 +1177,7 @@ void ImprimeCabeceraPoligFichero(const polig* poli,
 \date 22 de septiembre de 2011: Corregido bug que hacía que, dependiendo del
       compilador y/o los flags de optimización en la compilación, se imprimiesen
       mal (con un signo menos delante) los valores Not-a-Number.
+\date 10 de mayo de 2020: Adición de los argumentos \em impAtr y \em formAtr.
 \note Esta función todavía no está probada.
 */
 void ImprimePoligFichero(const polig* poli,
@@ -1080,6 +1192,8 @@ void ImprimePoligFichero(const polig* poli,
                          const int impLim,
                          const int impArea,
                          const char formArea[],
+                         const int impAtr,
+                         const char formAtr[],
                          FILE* idFich);
 /******************************************************************************/
 /******************************************************************************/
