@@ -1285,20 +1285,23 @@ int CalcAreaPolig(polig* poli,
                 }
             }
         }
+        //calculamos las superficies de todos los polígonos
+        //pongo la orden aquí para evitar un warning en mkoctfile
+        AreaPoligonosSimplesPolig(x,y,1,1,poli->posIni,poli->nVert,poli->nPolig,
+                                  0,poli->area);
     }
     else
     {
         //no hace falta copiarlas a los vectores auxiliares, sólo enlazarlas
         x = poli->x;
         y = poli->y;
+        //calculamos las superficies de todos los polígonos
+        //pongo la orden aquí para evitar un warning en mkoctfile
+        AreaPoligonosSimplesPolig(x,y,1,1,poli->posIni,poli->nVert,poli->nPolig,
+                                  0,poli->area);
     }
     ////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////
-    //calculamos las superficies de todos los polígonos
-    AreaPoligonosSimplesPolig(x,y,1,1,poli->posIni,poli->nVert,poli->nPolig,0,
-                              poli->area);
-    ////////////////////////////////////////////////////////////////////////////
-    ////////////////////////////////////////////////////////////////////////////
     //liberamos la memoria utilizada
     if(geo||(facCoor!=1.0))
     {
