@@ -7,12 +7,12 @@
 \author José Luis García Pallero, jgpallero@gmail.com
 \note Este fichero contiene funciones paralelizadas con OpenMP.
 \date 03 de junio de 2011
-\section Licencia Licencia
-Copyright (c) 2011, José Luis García Pallero. All rights reserved.
-
+\copyright
+Copyright (c) 2011-2020, José Luis García Pallero. All rights reserved.
+\par
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
-
+\par
 - Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this
@@ -21,7 +21,7 @@ are permitted provided that the following conditions are met:
 - Neither the name of the copyright holders nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.
-
+\par
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -379,7 +379,7 @@ polil* CreaPolil(const double* x,
     //paralelización con OpenMP
 #if defined(_OPENMP)
 #pragma omp parallel sections default(none) \
- shared(posNanX,x,nNanX,posNanY,y,nNanY)
+ shared(posNanX,nElem,incX,x,nNanX,posNanY,incY,y,nNanY)
 #endif
 {
 #if defined(_OPENMP)
@@ -1010,7 +1010,7 @@ void EscalaYTrasladaPolil(polil* poli,
     //paralelización con OpenMP
 #if defined(_OPENMP)
 #pragma omp parallel sections default(none) \
- shared(poli)
+ shared(poli,escalaX,trasladaX,escalaY,trasladaY)
 #endif
 {
 #if defined(_OPENMP)
@@ -1055,7 +1055,7 @@ void TrasladaYEscalaPolil(polil* poli,
     //paralelización con OpenMP
 #if defined(_OPENMP)
 #pragma omp parallel sections default(none) \
- shared(poli)
+ shared(poli,escalaX,trasladaX,escalaY,trasladaY)
 #endif
 {
 #if defined(_OPENMP)
@@ -1119,20 +1119,27 @@ void MuevePolil(polil* poli,
 /******************************************************************************/
 /******************************************************************************/
 int AligeraPolil(polil* poli,
+                 const int esf,
+                 const double facCoor,
                  const double tol,
+                 const int paralelizaTol,
                  const enum GEOC_DPEUCKER_ROBUSTO robusto,
-                 const size_t nPtosRobusto,
-                 const size_t nSegRobusto)
+                 const size_t nSegRobOrig,
+                 const size_t nSegRobAuto)
 {
     //índices para recorrer bucles
     size_t i=0,j=0;
     //posición inicial de la polilínea de trabajo y número de puntos aligerados
-    size_t posIni=0,nPtos=0;
+    size_t posIni=0,nVert=0,nPtos=0;
     //polilínea que es un mismo punto
     int pmp=0;
-    //coordenadas de las polilíneas aligeradas
+    //coordenadas de trabajo
     double* x=NULL;
     double* y=NULL;
+    double* xF=NULL;
+    double* yF=NULL;
+    //vértices de la polilínea de entrada más larga
+    size_t nVertMax=0;
     //estructura auxiliar
     polil* aux=NULL;
     //vector de posiciones después del aligerado
@@ -1161,20 +1168,65 @@ int AligeraPolil(polil* poli,
     }
     ////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////
+    //comprobamos si hay factor de escala para las coordenadas
+    if(facCoor!=1.0)
+    {
+        //vértices de la polilínea más larga
+        nVertMax = MaximoSizeT(poli->nVert,poli->nPolil,1);
+        //asignamos memoria para las coordenadas con factor de escala
+        xF = (double*)malloc(nVertMax*sizeof(double));
+        yF = (double*)malloc(nVertMax*sizeof(double));
+        //comprobamos posibles errores
+        if((xF==NULL)||(yF==NULL))
+        {
+            //liberamos la memoria asignada
+            LibMemPolil(aux);
+            free(xF);
+            free(yF);
+            //mensaje de error
+            GEOC_ERROR("Error de asignación de memoria");
+            //salimos de la función
+            return GEOC_ERR_ASIG_MEMORIA;
+        }
+    }
+    ////////////////////////////////////////////////////////////////////////////
+    ////////////////////////////////////////////////////////////////////////////
     //recorremos el número de polilíneas almacenadas
     for(i=0;i<poli->nPolil;i++)
     {
         //extraemos la posición inicial de la polilínea de trabajo
         posIni = poli->posIni[i];
+        //número de vértices de la polilínea
+        nVert = poli->nVert[i];
+        //comprobamos si hay factor de escala para las coordenadas
+        if(facCoor!=1.0)
+        {
+            //copiamos las coordenadas y aplicamos el factor de escala
+            for(j=0;j<nVert;j++)
+            {
+                xF[j] = facCoor*(poli->x[posIni+j]);
+                yF[j] = facCoor*(poli->y[posIni+j]);
+            }
+        }
+        else
+        {
+            //las coordenadas de la polilínea se quedan como están
+            xF = &(poli->x[posIni]);
+            yF = &(poli->y[posIni]);
+        }
         //aligeramos la polilínea de trabajo
-        pos = AligeraPolilinea(&(poli->x[posIni]),&(poli->y[posIni]),
-                               poli->nVert[i],1,1,tol,robusto,nPtosRobusto,
-                               nSegRobusto,&nPtos);
+        pos = AligeraPolilinea(xF,yF,nVert,1,1,tol,paralelizaTol,robusto,
+                               nSegRobOrig,nSegRobAuto,esf,&nPtos);
         //comprobamos posibles errores
         if(pos==NULL)
         {
             //liberamos la memoria asignada
             LibMemPolil(aux);
+            if(facCoor!=1.0)
+            {
+                free(xF);
+                free(yF);
+            }
             //mensaje de error
             GEOC_ERROR("Error de asignación de memoria");
             //salimos de la función
@@ -1183,7 +1235,7 @@ int AligeraPolil(polil* poli,
         //comprobamos si la polilínea se ha quedado en dos puntos
         if(nPtos==2)
         {
-            //comprobamos si los tres puntos son colineales
+            //comprobamos si los dos puntos son el mismo
             pmp = ((poli->x[posIni+pos[0]])==(poli->x[posIni+pos[1]]))&&
                   ((poli->y[posIni+pos[0]])==(poli->y[posIni+pos[1]]));
         }
@@ -1191,8 +1243,7 @@ int AligeraPolil(polil* poli,
         //no sea un único punto
         if((nPtos>2)||((nPtos==2)&&(!pmp)))
         {
-            //asignamos memoria para los vectores de coordenadas de la polilínea
-            //aligerada
+            //memoria para los vectores de coordenadas de la polilínea aligerada
             x = (double*)malloc(nPtos*sizeof(double));
             y = (double*)malloc(nPtos*sizeof(double));
             //comprobamos posibles errores
@@ -1203,6 +1254,11 @@ int AligeraPolil(polil* poli,
                 free(pos);
                 free(x);
                 free(y);
+                if(facCoor!=1.0)
+                {
+                    free(xF);
+                    free(yF);
+                }
                 //mensaje de error
                 GEOC_ERROR("Error de asignación de memoria");
                 //salimos de la función
@@ -1226,6 +1282,11 @@ int AligeraPolil(polil* poli,
                 free(pos);
                 free(x);
                 free(y);
+                if(facCoor!=1.0)
+                {
+                    free(xF);
+                    free(yF);
+                }
                 //escribimos el mensaje de error
                 GEOC_ERROR("Error de asignación de memoria");
                 //salimos de la función
@@ -1250,6 +1311,11 @@ int AligeraPolil(polil* poli,
         {
             //liberamos la memoria asignada
             LibMemPolil(aux);
+            if(facCoor!=1.0)
+            {
+                free(xF);
+                free(yF);
+            }
             //mensaje de error
             GEOC_ERROR("Error de asignación de memoria");
             //salimos de la función
@@ -1261,9 +1327,15 @@ int AligeraPolil(polil* poli,
     //enlazamos los campos de la estructura auxiliar a los de la estructura de
     //salida
     EnlazaCamposPolil(aux,poli);
-    //liberamos la memoria asignada a la estructura auxiliar (pero no a sus
-    //campos)
+    ////////////////////////////////////////////////////////////////////////////
+    ////////////////////////////////////////////////////////////////////////////
+    //liberamos la memoria asignada
     free(aux);
+    if(facCoor!=1.0)
+    {
+        free(xF);
+        free(yF);
+    }
     ////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////
     //salimos de la función
